'''
@author: Marius Lindauer
@license: GPLv3
'''

import os
import sys
import argparse
from argparse import ArgumentDefaultsHelpFormatter
import operator
import itertools
import matplotlib
matplotlib.use('Agg')
from matplotlib.pyplot import tight_layout, figure, subplots_adjust, subplot, savefig, show, setp
import matplotlib.gridspec
import numpy as np
import logging
import json
from scipy.stats.mstats import gmean

from permutationTester import PermutationTester

class EvaluateCSSC(object):
    '''
    classdocs
    '''


    def __init__(self, args):
        '''
        Constructor
        '''
        
        self.args = args
        #self.cutoff = 300 # 5000
        self.cutoff = 5000
        
        self.N_PERMUTATIONS = 100000 # 100000
        
        self.mapping_solvers = { "minisat-HACK-999ED-CSSC": "minisathack",
                            "Riss-4.27" : "riss",
                            "YalSAT" : "yalsat",
                            "clasp-3.0.4-p8" : "clasp",
                            "cryptominisat" : "cryptominisat",
                            "lingeling": "lingeling",
                            "SparrowToRiss" : "sparrow",
                            "DCCASat+march-rw" : "dccsat",
                            "CSCCSat2014" : "csccsat",
                            "probSAT" : "probsat"
                           }
        
        self.mapping_benchmarks = {"CSSC-Queens-300s-2day" : "queens",
                              "CSSC-GI-300s-2day" : "gi",
                              "CSSC-LABS-300s-2day": "labs",
                              "CSSC-unsat-unif-k5-300s-2day": "unif",
                              "CSSC-K3-300s-2day": "kthree",
                              "CSSC-3cnf-v350-300s-2day": "threecnf",
                              "CSSC-5SAT500-sat-300s-2day": "fivesatfiveh",
                              "CSSC-7SAT90-sat-300s-2day": "sevensatninety",
                              "CSSC-3SAT1k-sat-300s-2day": "threesatonek",
                              "CSSC-BMC08-300s-2day": "bmc",
                              "CSSC-CircuitFuzz-300s-2day": "curcit",
                              "CSSC-IBM-300s-2day": "hw",
                              "CSSC-SWV-300s-2days" : "swv"
                              }
        
        self.mapping_tracks = {"industrial" : "indu",
                          "crafted" : "crafted",
                          "random" : "random",
                          "randomSAT" : "randomsat"
                          }
                
    def main(self):
        ''' main method '''
        
        main_dict = self.read_csv()
        
        for track in ["industrial", "crafted", "random", "randomSAT"]:
            try:
                track_dict =  main_dict[track]
            except KeyError:
                continue # skip if not all tracks were evaluated
            #print("\\subsection{\%s}" %(mapping_tracks[track]))
            if self.args.tables:
                sorted_solvers = self.print_latex_tables(track, track_dict)
            if self.args.scatter:
                self.generate_scatter_plots(track, track_dict)
        if self.args.speeduptables:
            self.print_speedup_tables(main_dict)
        if self.args.allspeeduptables:
            self.print_all_speedup_tables(main_dict)
            
        if self.args.plotspeedup:
            self.speedup_params_plot(main_dict)
            
            
        #=======================================================================
        # for t_dct in main_dict.values():
        #     for d_dct in t_dct.values():
        #         for s_dct in d_dct.values():
        #               for c_dct in s_dct.values():
        #                   if c_dct.get("Performance"):
        #                       del c_dct["Performance"]
        #                   else:
        #                       for sp_dct in c_dct.values():
        #                           del sp_dct["Performance"]
        #        
        # logging.debug("%s" %(json.dumps(main_dict, indent=2))) 
        #=======================================================================
        
            #print("\\cearpage")
            #find_incumbents(track, track_dict, sorted_solvers)
                
    def read_csv(self):
        '''  read csv files and build main_dict 
                track -> 
                    scenario -> 
                        solver_dict -> 
                            configurator/default/default-train ->  
                                "training_perf"/"test_perf" -> 
                                    "PAR1"/"PAR10"/"TOs"/"Performance" '''
        
        main_dict = {}
        for file_ in os.listdir(self.args.dir):
            logging.debug(os.path.join(self.args.dir, file_))
            if not file_.endswith(".csv"):
                continue
            
            try:
                track, scen, solver, configurator, traintest = file_.replace(".csv","").split("_")
            except:
                logging.warn("Could not parse file name: %s" %(file_))
                continue
                                                                                              
            
            main_dict[track] = main_dict.get(track,{})
            main_dict[track][scen] = main_dict[track].get(scen, {})
            main_dict[track][scen][solver] = main_dict[track][scen].get(solver, {"default-train": {}, "default": {}})
            main_dict[track][scen][solver][configurator] = main_dict[track][scen][solver].get(configurator, {"training_perf": {}, "test_perf": {}})
            
            if traintest == "TRAIN":
                p_dict = main_dict[track][scen][solver][configurator]["training_perf"]
                t_dict = main_dict[track][scen][solver]["default-train"]
            else:
                p_dict = main_dict[track][scen][solver][configurator]["test_perf"]
                t_dict = main_dict[track][scen][solver]["default"]
            
            inst_perf_dict = {}
            inst_default_dict = {}
            
            with open(os.path.join(self.args.dir, file_)) as fp:
                fp.readline() # ignore header
                for line in fp:
                    inst, def_, conf_ = line.split(",")
                    inst_perf_dict[inst] = float(conf_)
                    inst_default_dict[inst] = float(def_)
            
            if not inst_perf_dict:
                continue 
            
            p_dict["Performance"] = inst_perf_dict
            tos, par1, par10 = self._get_stats(inst_perf_dict)
            tos_d, par1_d, par10_d = self._get_stats(inst_default_dict)
            
            def_, conf_ = self.extract_values_in_order(inst_default_dict, inst_perf_dict, inst_default_dict.keys())
            pairs = filter(lambda x: x[0] < self.cutoff or x[1] < self.cutoff, zip(def_, conf_))
            speedup = 1
            if pairs: # if all instances were unsolved, skip
                    default = sum(map(lambda x: x if x < self.cutoff else self.cutoff*10, map(lambda x: x[0], pairs)))
                    configured = sum(map(lambda x: x if x < self.cutoff else self.cutoff*10, map(lambda x: x[1], pairs)))
                    speedup = default/configured
            
            p_dict["PAR1"] = par1
            p_dict["PAR10"] = par10
            p_dict["TOs"] = tos
            p_dict["Speedup"] = speedup
            
            t_dict["Performance"] = inst_default_dict
            t_dict["PAR1"] = par1_d
            t_dict["PAR10"] = par10_d
            t_dict["TOs"] = tos_d
            
        return main_dict
                        
    def _get_stats(self, inst_perf_dict):
        '''given a dictionary of performance values, returns timeouts, PAR1 and PAR10 '''
        values = inst_perf_dict.values()
        
        par1_values = map(lambda x: x if x < self.cutoff else self.cutoff, values)
        par1 = sum(par1_values) / len(par1_values)
        par10_values = map(lambda x: x if x < self.cutoff else self.cutoff*10, values)
        par10 = sum(par10_values) / len(par10_values)
        tos_values = map(lambda x : 0 if x < self.cutoff else 1, values)
        tos = sum(tos_values)
        
        return tos, par1, par10
    
    def print_speedup_tables(self, main_dict):
        ''' print table: configurator x solver with geometric speedups'''
        #=======================================================================
        #         track -> 
        #             scenario -> 
        #                 solver_dict -> 
        #                     configurator/default/default-train ->  
        #                         "training_perf"/"test_perf" -> 
        #                             "PAR1"/"PAR10"/"TOs"/"Performance" 
        # 
        #=======================================================================
 
        solver_speedup = {} # solver -> config -> train/test -> speedups
        solver_slowdown = {}
        
        for trac_dict in main_dict.itervalues():
            for scen_dict in trac_dict.itervalues():
                for solver, solver_dict in scen_dict.iteritems():
                    smac_cont_runs = dict(filter(lambda x: "smac" in x[0] and not "smac-disc" in x[0], solver_dict.items()))
                    smac_disc_runs = dict(filter(lambda x: "smac-disc" in x[0], solver_dict.items()))
                    gga_cont_runs = dict(filter(lambda x: "gga" in x[0] and not "gga-disc" in x[0], solver_dict.items()))
                    gga_disc_runs = dict(filter(lambda x: "gga-disc" in x[0], solver_dict.items()))
                    pils_runs = dict(filter(lambda x: "paramils" in x[0], solver_dict.items()))
                    
                    best_smac_cont = self.get_best_train(smac_cont_runs)
                    best_smac_disc = self.get_best_train(smac_disc_runs)
                    best_gga_cont = self.get_best_train(gga_cont_runs)
                    best_gga_disc = self.get_best_train(gga_disc_runs)
                    best_pils = self.get_best_train(pils_runs)
                    
                    best_overall = self.get_best_train(solver_dict)
                    
                    solver_speedup[solver] = solver_speedup.get(solver, {"SMAC-disc": {"test":[], "train":[]}, 
                                                                         "SMAC-cont": {"test":[], "train":[]},
                                                                         "GGA-disc": {"test":[], "train":[]}, 
                                                                         "GGA-cont": {"test":[], "train":[]},
                                                                         "ParamILS": {"test":[], "train":[]}})

                    solver_slowdown[solver] = solver_slowdown.get(solver, {"SMAC-disc": {"test":[], "train":[]}, 
                                                                         "SMAC-cont": {"test":[], "train":[]},
                                                                         "GGA-disc": {"test":[], "train":[]}, 
                                                                         "GGA-cont": {"test":[], "train":[]},
                                                                         "ParamILS": {"test":[], "train":[]}})

                    
                    solver_slowdown[solver]["SMAC-disc"]["test"].append(
                                            solver_dict[best_overall]["test_perf"]["Speedup"] / solver_dict[best_smac_disc]["test_perf"]["Speedup"] )
                    solver_slowdown[solver]["SMAC-disc"]["train"].append(
                                            solver_dict[best_overall]["training_perf"]["Speedup"] / solver_dict[best_smac_disc]["training_perf"]["Speedup"] )
                    solver_slowdown[solver]["SMAC-cont"]["test"].append(
                                            solver_dict[best_overall]["test_perf"]["Speedup"] / solver_dict[best_smac_cont]["test_perf"]["Speedup"] )
                    solver_slowdown[solver]["SMAC-cont"]["train"].append(
                                            solver_dict[best_overall]["training_perf"]["Speedup"] / solver_dict[best_smac_cont]["training_perf"]["Speedup"] )
                    try:
                        solver_slowdown[solver]["GGA-disc"]["test"].append(
                                             solver_dict[best_overall]["test_perf"]["Speedup"] / solver_dict[best_gga_disc]["test_perf"]["Speedup"] ) 
                        solver_slowdown[solver]["GGA-disc"]["train"].append(
                                             solver_dict[best_overall]["training_perf"]["Speedup"] / solver_dict[best_gga_disc]["training_perf"]["Speedup"] )
                    except KeyError:
                        solver_slowdown[solver]["GGA-disc"]["test"].append(0)
                        solver_slowdown[solver]["GGA-disc"]["train"].append(0)

                    try:
                        solver_slowdown[solver]["GGA-cont"]["test"].append(
                                             solver_dict[best_overall]["test_perf"]["Speedup"] / solver_dict[best_gga_cont]["test_perf"]["Speedup"] )
                        solver_slowdown[solver]["GGA-cont"]["train"].append(
                                             solver_dict[best_overall]["training_perf"]["Speedup"] / solver_dict[best_gga_cont]["training_perf"]["Speedup"] )
                    except KeyError:
                        solver_slowdown[solver]["GGA-cont"]["test"].append(0)
                        solver_slowdown[solver]["GGA-cont"]["train"].append(0)

                    solver_slowdown[solver]["ParamILS"]["test"].append(
                                             solver_dict[best_overall]["test_perf"]["Speedup"] / solver_dict[best_pils]["test_perf"]["Speedup"] )
                    solver_slowdown[solver]["ParamILS"]["train"].append(
                                             solver_dict[best_overall]["training_perf"]["Speedup"] / solver_dict[best_pils]["training_perf"]["Speedup"] )


                    
                    if solver == "lingeling":
                        print solver_slowdown[solver]
                        exit
                    

                    
                    solver_speedup[solver]["SMAC-disc"]["test"].append(
                                            solver_dict[best_smac_disc]["test_perf"]["Speedup"])
                    solver_speedup[solver]["SMAC-disc"]["train"].append(
                                            solver_dict[best_smac_disc]["training_perf"]["Speedup"])
                    solver_speedup[solver]["SMAC-cont"]["test"].append(
                                            solver_dict[best_smac_cont]["test_perf"]["Speedup"])
                    solver_speedup[solver]["SMAC-cont"]["train"].append(
                                            solver_dict[best_smac_cont]["training_perf"]["Speedup"])
                    try:
                        solver_speedup[solver]["GGA-disc"]["test"].append(
                                             solver_dict[best_gga_disc]["test_perf"]["Speedup"])
                        solver_speedup[solver]["GGA-disc"]["train"].append(
                                             solver_dict[best_gga_disc]["training_perf"]["Speedup"])
                    except KeyError:
                        solver_speedup[solver]["GGA-disc"]["test"].append(1)
                        solver_speedup[solver]["GGA-disc"]["train"].append(1)

                    try:
                        solver_speedup[solver]["GGA-cont"]["test"].append(
                                             solver_dict[best_gga_cont]["test_perf"]["Speedup"])
                        solver_speedup[solver]["GGA-cont"]["train"].append(
                                             solver_dict[best_gga_cont]["training_perf"]["Speedup"])
                    except KeyError:
                        solver_speedup[solver]["GGA-cont"]["test"].append(1)
                        solver_speedup[solver]["GGA-cont"]["train"].append(1)

                    solver_speedup[solver]["ParamILS"]["test"].append(
                                             solver_dict[best_pils]["test_perf"]["Speedup"])
                    solver_speedup[solver]["ParamILS"]["train"].append(
                                             solver_dict[best_pils]["training_perf"]["Speedup"])

        print("\\begin{table}[th]")
        print("\sffamily\small\centering")
        print("\setlength\\tabcolsep{0.3em}")
        print("\\begin{tabular}{l | ccccc | ccccc}")
        print("\\toprule[1pt]")
        print(" & \\multicolumn{5}{c|}{Training} & \\multicolumn{5}{c}{Test}\\\\")
        print(" & \\smac{}-d & \\smac{}-c & \\pils{} & \\gga{}-d & \\gga{}-c & \\smac{}-d & \\smac{}-c & \\pils{} & \\gga{}-d & \\gga{}-c  \\\\")
        print("\\midrule")

        for solver in sorted(solver_slowdown.keys()):
            s_dict = solver_slowdown[solver]
            print("\\%s & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$\\\\" %(self.mapping_solvers[solver],
                                                                       gmean(s_dict["SMAC-disc"]["train"]),
                                                                       gmean(s_dict["SMAC-cont"]["train"]),
                                                                       gmean(s_dict["ParamILS"]["train"]),
                                                                       gmean(s_dict["GGA-disc"]["train"]),
                                                                       gmean(s_dict["GGA-cont"]["train"]),
                                                                       gmean(s_dict["SMAC-disc"]["test"]),
                                                                       gmean(s_dict["SMAC-cont"]["test"]),
                                                                       gmean(s_dict["ParamILS"]["test"]),
                                                                       gmean(s_dict["GGA-disc"]["test"]),
                                                                       gmean(s_dict["GGA-cont"]["test"])
                                                                       ))
        print("\\bottomrule[1pt]")
        print("\end{tabular}")
#        print("\caption{CSSC'14: Geometric mean of speedups for each solver configured by different configurators}")
        print("\caption{CSSC'14: Geometric mean of slowdowns over using best configurator}")
        print("\\end{table}")
        
    def print_all_speedup_tables(self, main_dict):
        ''' print tables with speedup for each scenario in latex style '''
        #=======================================================================
        #         track -> 
        #             scenario -> 
        #                 solver_dict -> 
        #                     configurator/default/default-train ->  
        #                         "training_perf"/"test_perf" -> 
        #                             "PAR1"/"PAR10"/"TOs"/"Performance" 
        # 
        #=======================================================================     
        solver_speedup = {} # solver -> config -> train/test -> speedups
        
        for track,trac_dict in main_dict.iteritems():
            
            print("\\begin{table}[th]")
            print("\sffamily\small\centering")
            print("\setlength\\tabcolsep{0.3em}")
            print("\\begin{tabular}{l | ccccc | ccccc}")
            print("\\toprule[1pt]")
            print(" & \\multicolumn{5}{c|}{Training} & \\multicolumn{5}{c}{Test}\\\\")
            print(" & \\smac{}-d & \\smac{}-c & \\pils{} & \\gga{}-d & \\gga{}-c & \\smac{}-d & \\smac{}-c & \\pils{} & \\gga{}-d & \\gga{}-c  \\\\")
            
            for scen,scen_dict in trac_dict.iteritems():
                print("\\midrule")
                print(" \\multicolumn{11}{c}{%s}\\\\" % (scen))
                print("\\midrule")
                
                
                for solver, solver_dict in scen_dict.iteritems():
                    smac_cont_runs = dict(filter(lambda x: "smac" in x[0] and not "smac-disc" in x[0], solver_dict.items()))
                    smac_disc_runs = dict(filter(lambda x: "smac-disc" in x[0], solver_dict.items()))
                    gga_cont_runs = dict(filter(lambda x: "gga" in x[0] and not "gga-disc" in x[0], solver_dict.items()))
                    gga_disc_runs = dict(filter(lambda x: "gga-disc" in x[0], solver_dict.items()))
                    pils_runs = dict(filter(lambda x: "paramils" in x[0], solver_dict.items()))
                    
                    best_smac_cont = self.get_best_train(smac_cont_runs)
                    best_smac_disc = self.get_best_train(smac_disc_runs)
                    best_gga_cont = self.get_best_train(gga_cont_runs)
                    best_gga_disc = self.get_best_train(gga_disc_runs)
                    best_pils = self.get_best_train(pils_runs)
        
                    
                    print("\\%s & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$ & $%.1f$\\\\" %(self.mapping_solvers[solver],
                                                                                                                                solver_dict[best_smac_disc]["training_perf"]["Speedup"],
                                                                                                                                solver_dict[best_smac_cont]["training_perf"]["Speedup"],
                                                                                                                                solver_dict[best_pils]["training_perf"]["Speedup"],
                                                                                                                                solver_dict.get(best_gga_disc,-1).get("training_perf").get("Speedup",-1) if solver_dict.get(best_gga_disc) else -1,
                                                                                                                                solver_dict.get(best_gga_cont,-1).get("training_perf").get("Speedup",-1) if solver_dict.get(best_gga_cont) else -1,
                                                                                                                                solver_dict[best_smac_disc]["test_perf"]["Speedup"],
                                                                                                                                solver_dict[best_smac_cont]["test_perf"]["Speedup"],
                                                                                                                                solver_dict[best_pils]["test_perf"]["Speedup"],
                                                                                                                                solver_dict.get(best_gga_disc,-1).get("test_perf").get("Speedup",-1) if solver_dict.get(best_gga_disc) else -1,
                                                                                                                                solver_dict.get(best_gga_cont,-1).get("test_perf").get("Speedup",-1) if solver_dict.get(best_gga_cont) else -1))
                    
            print("\\bottomrule[1pt]")
            print("\end{tabular}")
            print("\caption{CSSC'14: \\%s}" %(self.mapping_tracks[track]))
            print("\\end{table}")                
                
                          
    def print_latex_tables(self, track, track_dict):
        ''' print results tables in latex style '''
        solvers_dict = {}
        # collect data for overall ranking:
        for scenario, scenario_dict in track_dict.items():
            for solver in scenario_dict.keys():
                solver_dict = scenario_dict[solver]
                solvers_dict[solver] = solvers_dict.get(solver, {"n": 0, "par1c": 0.0, "par10c": 0.0, "tosc": 0, 
                                                                 "par1d": 0.0, "par10d": 0.0, "tosd": 0,
                                                                 "par1c-train": 0.0, "par10c-train": 0.0, "tosc-train": 0, 
                                                                 "par1d-train": 0.0, "par10d-train": 0.0, "tosd-train": 0 })
                s_dict = solvers_dict[solver] 
                print(scenario, solver)
                best_configurator = self.get_best_train(solver_dict)
                if not best_configurator:
                    continue
                try: #TODO: fix
                    s_dict["par1c"] += solver_dict[best_configurator]["test_perf"]["PAR1"]
                except KeyError:
                    continue
                s_dict["par10c"] += solver_dict[best_configurator]["test_perf"]["PAR10"]
                s_dict["tosc"] += solver_dict[best_configurator]["test_perf"]["TOs"]
                s_dict["par1d"] += solver_dict["default"]["PAR1"]
                s_dict["par10d"] += solver_dict["default"]["PAR10"]
                s_dict["tosd"] += solver_dict["default"]["TOs"]
                s_dict["n"] += 1
                
                s_dict["par1c-train"] += solver_dict[best_configurator]["training_perf"]["PAR1"]
                s_dict["par10c-train"] += solver_dict[best_configurator]["training_perf"]["PAR10"]
                s_dict["tosc-train"] += solver_dict[best_configurator]["training_perf"]["TOs"]
                s_dict["par1d-train"] += solver_dict["default-train"]["PAR1"] 
                s_dict["par10d-train"] += solver_dict["default-train"]["PAR10"]
                s_dict["tosd-train"] += solver_dict["default-train"]["TOs"]
    
        # sort solvers by overall ranking
        sorted_solvers = self.sorted_overall(solvers_dict)
        
        for scenario, scenario_dict in track_dict.items():
            # print table per track
            print("\\begin{table}[h]")
            print("\\footnotesize")
            print("\hspace{-1cm}")
            print("\\begin{tabular}{l | cccc | cccc | l}")
            print("\hline\hline")
            print(" & \multicolumn{4}{c|}{Training performance} & \multicolumn{4}{c|}{Test performance} & \\\\")
            print("Solver & \multicolumn{2}{c}{$\#$Timeouts}  & \multicolumn{2}{c|}{PAR1} & \multicolumn{2}{c}{$\#$Timeouts}  & \multicolumn{2}{c|}{PAR1} & Configurator\\\\")
            #print("\cline{2-3} \cline{4-5} \cline{6-7} \cline{8-9}")
            print(" & default & config. & default & config. & default & config. & default & config. & \\\\ ")
            print("\hline")
            
            for solver in sorted_solvers:
                print("%s %s" %(scenario, solver))
                solver_dict = scenario_dict[solver]
    
                best_configurator = self.get_best_train(solver_dict)
                if not best_configurator:
                    continue
                par1_default_train = solver_dict["default-train"]["PAR1"]
                par1_config_train = solver_dict[best_configurator]["training_perf"]["PAR1"]
                par1_default_train_dic = solver_dict["default-train"]["Performance"]
                par1_config_train_dic = solver_dict[best_configurator]["training_perf"]["Performance"]
                par1_train_string = self.generate_table_string(par1_default_train, par1_config_train, par1_default_train_dic, par1_config_train_dic, 1, 300)
                
                tos_default = solver_dict["default"]["TOs"]
                try:
                    tos_config = solver_dict[best_configurator]["test_perf"]["TOs"]
                except KeyError:
                    print("%s %s %s %.2f" %(solver, scenario, best_configurator, solver_dict[best_configurator]["training_perf"]["PAR10"] ))
                    continue
                tos_default_dic = solver_dict["default"]["Performance"]
                tos_config_dic = solver_dict[best_configurator]["test_perf"]["Performance"]
                tos_test_string = self.generate_table_string(tos_default,tos_config,tos_default_dic, tos_config_dic, -1, 300)
                
                par1_default = solver_dict["default"]["PAR1"]
                par1_config = solver_dict[best_configurator]["test_perf"]["PAR1"]
                par1_default_dic = solver_dict["default"]["Performance"]
                par1_config_dic = solver_dict[best_configurator]["test_perf"]["Performance"]
                par1_test_string = self.generate_table_string(par1_default, par1_config, par1_default_dic, par1_config_dic, 1, 300)
                
                tos_default_train = solver_dict["default-train"]["TOs"]
                tos_config_train = solver_dict[best_configurator]["training_perf"]["TOs"]    
                tos_default_train_dic = solver_dict["default-train"]["Performance"]
                tos_config_train_dic = solver_dict[best_configurator]["training_perf"]["Performance"]
                tos_train_string = self.generate_table_string(tos_default_train, tos_config_train, tos_default_train_dic, tos_config_train_dic, -1, 300)
                    
                    
                print("\%s & %s & %s & %s & %s & %s\\\\" %(self.mapping_solvers.get(solver,solver), 
                                                           tos_train_string,par1_train_string,
                                                           tos_test_string, par1_test_string,
                                                           best_configurator))
                
            print("\hline\hline")
            print("\end{tabular}")
            print("\caption{Track: \%s; Benchmarks: \%s}" %(self.mapping_tracks[track], self.mapping_benchmarks[scenario]))
            print("\\end{table}")
        
        # overall results
        print("\\begin{table}[th]")
        print("\sffamily\small\centering")
        print("\setlength\\tabcolsep{0.3em}")
        print("\\begin{tabular}{l | cccc | cccc | cc}")
        print("\hline\hline")
        print(" & \multicolumn{4}{c|}{Training performance} & \multicolumn{4}{c|}{Test performance} &  & \\\\")
        print("Solver & \multicolumn{2}{c}{$\#$Timeouts}  & \multicolumn{2}{c|}{avg. time} & \multicolumn{2}{c}{$\#$Timeouts}  & \multicolumn{2}{c|}{avg. time} & \multicolumn{2}{c}{Rank}\\\\")
        #print("\cline{2-3} \cline{4-5} \cline{6-7} \cline{8-9}")
        print(" & def & conf & def & conf & def & conf & def & conf & def & CSSC\\\\ ")
        print("\hline")
        
        default_perf = list((solver, solvers_dict[solver]["tosd"], solvers_dict[solver]["par1d"]) for solver in sorted_solvers)
        default_ranks = sorted(default_perf, key=operator.itemgetter(1,2))
        default_ranks = map(lambda x: x[0], default_ranks)
            
        for rank, solver in enumerate(sorted_solvers):
            stats = solvers_dict[solver]
            #if stats["n"] == 3:
            str_ = ["\%s" %(self.mapping_solvers.get(solver,solver))]
            if stats["tosd-train"] > stats["tosc-train"]:
                str_.append("$%d$ & $\mathbf{%d}$" %(stats["tosd-train"], stats["tosc-train"]))
            elif stats["tosd-train"] < stats["tosc-train"]:
                str_.append("$\mathbf{%d}$ & $%d$" %(stats["tosd-train"], stats["tosc-train"]))
            else:
                str_.append("$%d$ & $%d$" %(stats["tosd-train"], stats["tosc-train"]))
            if stats["par1d-train"]/stats["n"] > stats["par1c-train"]/stats["n"]:
                str_.append("$%.1f$ & $\mathbf{%.1f}$" %(stats["par1d-train"]/stats["n"], stats["par1c-train"]/stats["n"]))
            elif stats["par1d-train"]/stats["n"] < stats["par1c-train"]/stats["n"]:
                str_.append("$\mathbf{%.1f}$ & $%.1f$" %(stats["par1d-train"]/stats["n"], stats["par1c-train"]/stats["n"]))
            else:
                str_.append("$%.1f$ & $%.1f$" %(stats["par1d-train"]/stats["n"], stats["par1c-train"]/stats["n"]))
            if stats["tosd"] > stats["tosc"]:
                str_.append("$%d$ & $\mathbf{%d}$" %(stats["tosd"], stats["tosc"]))
            elif stats["tosd"] < stats["tosc"]:
                str_.append("$\mathbf{%d}$ & $%d$" %(stats["tosd"], stats["tosc"]))
            else:
                str_.append("$%d$ & $%d$" %(stats["tosd"], stats["tosc"]))
            if stats["par1d"]/stats["n"] > stats["par1c"]/stats["n"]:
                str_.append("$%.1f$ & $\mathbf{%.1f}$" %(stats["par1d"]/stats["n"], stats["par1c"]/stats["n"]))
            elif stats["par1d"]/stats["n"] < stats["par1c"]/stats["n"]:
                str_.append("$\mathbf{%.1f}$ & $%.1f$" %(stats["par1d"]/stats["n"], stats["par1c"]/stats["n"]))
            else:
                str_.append("$%.1f$ & $%.1f$" %(stats["par1d"]/stats["n"], stats["par1c"]/stats["n"]))
            
            str_.append("%d & %d\\\\" %(
                            default_ranks.index(solver)+1,
                            rank+1
                            ))
            if rank % 2 == 1:
                print("\myrowcolour{}%s" %(" & ".join(str_)))
            else:
                print(" & ".join(str_))
        print("\hline\hline")
        print("\end{tabular}")
        print("\caption{Overall results for track: \%s; ranked by number of timeouts of configured solvers on test sets (tie-breaker: PAR1). Bold face indicates that $x$ has better performance than $y$.}" %(self.mapping_tracks[track]))
        print("\\end{table}")
        
        
        ############################
        # FINAL TABLE              #
        ############################
        
        # track 
        #       -> scenario 
        #         -> solver_dict 
        #           -> configurator/default/default-test 
        #             ->  "training_perf"/"test_perf" 
        #               -> "PAR1"/"PAR10"/"TOs"/"Performance"
        
            # overall results
        print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
        print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
        scens = sorted(track_dict.keys())
        print("\\begin{table}[th]")
        print("\\setlength\\tabcolsep{0.4em}")
        print("\\sffamily\\footnotesize\\centering")
        print("\\begin{tabular}{l | %s | c | cc}" %("c"*(len(track_dict))))
        print("\\toprule[1.0pt]")
        print(" & \multicolumn{%d}{c|}{$\#$timeouts default $\\to$ $\#$ timeouts configured (on test set)} & \multicolumn{2}{c}{Rank} \\\\" %(len(track_dict)+1))
        print(" & %s & \\textit{Overall} & def & {\\scriptsize{CSSC}}\\\\" %(" & ".join(map(lambda x: "\\"+self.mapping_benchmarks[x], scens))))
        #print("\cline{2-3} \cline{4-5} \cline{6-7} \cline{8-9}")
        print("\\midrule")
        
        default_perf = list((solver, solvers_dict[solver]["tosd"], solvers_dict[solver]["par1d"]) for solver in sorted_solvers)
        default_ranks = sorted(default_perf, key=operator.itemgetter(1,2))
        default_ranks = map(lambda x: x[0], default_ranks)
            
        for rank, solver in enumerate(sorted_solvers):
            stats = solvers_dict[solver]
            #if stats["n"] == 3:
            str_ = ["\%s" %(self.mapping_solvers.get(solver,solver))]
    
            for scen in scens:
                solver_dict = track_dict[scen][solver]
                
                best_configurator = self.get_best_train(solver_dict)
    
                if solver_dict["default"]["TOs"] > solver_dict[best_configurator]["test_perf"]["TOs"]:
                    str_.append("$%d \\to \mathbf{%d}$" %(solver_dict["default"]["TOs"], solver_dict[best_configurator]["test_perf"]["TOs"]))
                elif solver_dict["default"]["TOs"] < solver_dict[best_configurator]["test_perf"]["TOs"]:
                    str_.append("$\mathbf{%d} \\to %d$" %(solver_dict["default"]["TOs"], solver_dict[best_configurator]["test_perf"]["TOs"]))
                else:
                    str_.append("$%d \\to %d$" %(solver_dict["default"]["TOs"], solver_dict[best_configurator]["test_perf"]["TOs"]))
            
            
            if stats["tosd"] > stats["tosc"]:
                str_.append("$%d \\to \mathbf{%d}$" %(stats["tosd"], stats["tosc"]))
            elif stats["tosd"] < stats["tosc"]:
                str_.append("$\mathbf{%d} \\to %d$" %(stats["tosd"], stats["tosc"]))
            else:
                str_.append("$%d \\to %d$" %(stats["tosd"], stats["tosc"]))
                
            str_.append("%d & %d\\\\" %(
                            default_ranks.index(solver)+1,
                            rank+1
                            ))
            if rank % 2 == 1:
                print("\myrowcolour{}%s" %(" & ".join(str_)))
            else:
                print(" & ".join(str_))
        print("\\bottomrule[1.0pt]")
        print("\end{tabular}")
        print("\caption{Results for CSSC 20XX competition track \\%s. For each solver and benchmark, we show the number of test set timeouts achieved with the default and the configured parameter setting, bold-facing the better one; ties were broken by the solver's average runtime (not shown for brevity). Results were aggregated across all benchmarks to compute the final ranking.}" %(self.mapping_tracks[track]))        
        print("\\end{table}")
        
        return sorted_solvers
    
    def sorted_overall(self, solvers_dict):
        ''' sorts solver by their performance '''
        solver_perf = {}
        for solver, solver_dict in solvers_dict.items():
            solver_perf[solver] = [solver_dict["tosc"],solver_dict["par1c"]]
            
        sorted_solvers = []
        for key, value1,in sorted(solver_perf.iteritems(), key=lambda (k,v): (v[0], v[1],k)):
            sorted_solvers.append(key)
        return sorted_solvers
       
    def get_best_train(self, solver_dict):
        ''' returns best configurator run '''
        best_perf = sys.maxint
        best_configurator = None
        for configurator, conf_dict in solver_dict.items():
            if configurator in ["default", "default-train", "test_perf"] :
                continue
            if best_perf > conf_dict["training_perf"]["PAR10"]:
                best_perf = conf_dict["training_perf"]["PAR10"]
                best_configurator = configurator
        return best_configurator
        
    def generate_table_string(self, val1, val2, dic1, dic2, parx, cutoff):
        ''' returns strings for latex tables with highlighting'''
        dominates, switch, pvalue = self.permutation_testing(dic1, dic2, parx, cutoff)
            
        if dominates and switch:
            if parx > 0:
                return "$%.2f$ & $\mathbf{%.2f}$" %(val1, val2)
            else:
                return "$%d$ & $\mathbf{%d}$" %(val1, val2)
        elif dominates and not switch:
            if parx > 0:
                return "$\mathbf{%.2f}$ & $%.2f$" %(val1, val2)
            else:
                return "$\mathbf{%d}$ & $%d$" %(val1, val2)
        elif not dominates:
            if parx > 0:
                return "$%.2f$ & $%.2f$" %(val1, val2)
            else:
                return "$%d$ & $%d$" %(val1, val2)
        
    def permutation_testing(self, dic1, dic2, parx, cutoff):
        ''' permutation test of two solvers 
            Arguments:
                 dic1: inst -> perf
                 dic2: inst -> perf
                 parx: factor for PAR scroes
                 cutoff: runtime cutoff
        '''
        
        tester = PermutationTester()
        vec1, vec2 = self.extract_values_in_order(dic1, dic2, dic1.keys()) 
        vec1 = tester.convert_to_PARX(vec1, cutoff, parx)
        vec2 = tester.convert_to_PARX(vec2, cutoff, parx)
        if self.args.permutationtest:
            dominates, switch, pvalue = tester.doTest(vec1, vec2, permutations=self.N_PERMUTATIONS, name1=str(parx), name2=str(parx))
        else:
            dominates = False
            switch = False
            pvalue = 0.0
        return dominates, switch, pvalue
    
    def extract_values_in_order(self, dic1, dic2, keys):
        ''' extract the values of two directories with the same keys in the same order and returns two lists '''
        vec1 = []
        vec2 = []
        for k in keys:
            vec1.append(dic1[k])
            vec2.append(dic2[k])
        return vec1, vec2

    def generate_scatter_plots(self, track, track_dict): #, sorted_solvers):
        for scenario, scenario_dict in track_dict.items():
            print("\\begin{figure}[h]")
            i = 0
            for solver in sorted(scenario_dict.iterkeys()):
                solver_dict = scenario_dict[solver]
                id = "%s_%s_%s.pdf" %(track, scenario, solver.replace(".","_")) 
                best_configurator = self.get_best_train(solver_dict)
                if not best_configurator:
                    continue
                i += 1
                performance_default = solver_dict["default"]["Performance"]
                performance_config = solver_dict[best_configurator]["test_perf"]["Performance"]
                instances = performance_default.keys()
                p_def = np.array([performance_default[inst_] for inst_ in instances])
                p_conf = np.array([performance_config[inst_] for inst_ in instances])
                # minimal meaningful runtime set to 0.005
                np.place(p_def, p_def<0.05, 0.05)
                np.place(p_conf, p_conf<0.05, 0.05)
                
                # set timeouts to 600 to see them better in plots
                np.place(p_def, p_def>=300, 600)
                np.place(p_conf, p_conf>=300, 600)
                
                min_val = min(min(p_def),min(p_conf))*0.5
                
                self.plot_scatter_plot(p_def, p_conf, ["Default in sec.", "Configured in sec."], min_val=0.01, max_val=300, save=id, grey_factor=2, linefactors=[2, 10,100])
                if i % 2 == 0:
                    print("\subfigure[\%s; TOs: $%d \\to %d$]{\includegraphics[width=0.55\\textwidth]{images/%s}}\\\\" %(self.mapping_solvers.get(solver,solver), 
                                                                                                                           solver_dict["default"]["TOs"],  
                                                                                                                           solver_dict[best_configurator]["test_perf"]["TOs"],
                                                                                                                           id))
                else:
                    print("\subfigure[\%s; TOs: $%d \\to %d$]{\includegraphics[width=0.55\\textwidth]{images/%s}}" %(self.mapping_solvers.get(solver,solver), 
                                                                                                                           solver_dict["default"]["TOs"],  
                                                                                                                           solver_dict[best_configurator]["test_perf"]["TOs"],
                                                                                                                           id))
            print("\caption{Track: \%s, Benchmarks: \%s}" %(self.mapping_tracks[track], self.mapping_benchmarks[scenario]))
            print("\\end{figure}")
            print("")
    
    def plot_scatter_plot(self, x_data, y_data, labels, title="", save="", debug=False,
                          min_val=None, max_val=1000, grey_factor=1, linefactors=None,
                          user_fontsize=20, dpi=100):
        regular_marker = 'x'
        timeout_marker = '+'
        grey_marker = 'x'
        c_angle_bisector = "#e41a1c"  # Red
        c_good_points = "#999999"     # Grey
        c_other_points = "k"
        size = 1
        st_ref = "--"
    
        ticklabel_size = user_fontsize
        linefactor_size = user_fontsize - 2
        label_size = user_fontsize + 1
    
        #------
        # maximum_value: location for timeout points
        # max_val      : Initially user-defined timeout, then set to axes limit
        # time_out_val : location for timeout points
        # -----
    
        maximum_value = max_val
    
        # Colors
        ref_colors = itertools.cycle([  # "#e41a1c",    # Red
                                     "#377eb8",    # Blue
                                     "#4daf4a",    # Green
                                     "#984ea3",    # Purple
                                     "#ff7f00",    # Orange
                                     "#ffff33",    # Yellow
                                     "#a65628",    # Brown
                                     "#f781bf",    # Pink
                                     # "#999999",    # Grey
                                     ])
    
        # Set up figure
        ratio = 1
        gs = matplotlib.gridspec.GridSpec(ratio, 1)
        fig = figure(1, dpi=100)
        fig.suptitle(title, fontsize=16)
        ax1 = subplot(gs[0:ratio, :], aspect='equal')
        ax1.grid(True, linestyle='-', which='major', color='lightgrey', alpha=0.5)
    
        # set initial limits
        if min_val is not None:
            auto_min_val = min([min(x_data), min(y_data), min_val])
        else:
            auto_min_val = min([min(x_data), min(y_data)])
        auto_max_val = maximum_value
        timeout_val = maximum_value *2 #+ 10**int((np.log10(max_val)))
    
        # Plot angle bisector and reference_lines
        out_up = auto_max_val
        out_lo = auto_min_val
    
        ax1.plot([out_lo, out_up], [out_lo, out_up], c=c_angle_bisector)
    
        if linefactors is not None:
            for f in linefactors:
                c = ref_colors.next()
                # Lower reference lines
                ax1.plot([f*out_lo, out_up], [out_lo, (1.0/f)*out_up], c=c,
                         linestyle=st_ref, linewidth=size*1.5)
                # Upper reference lines
                ax1.plot([out_lo, (1.0/f)*out_up], [f*out_lo, out_up], c=c,
                         linestyle=st_ref, linewidth=size*1.5)
                offset = 4.4
                if int(f) == f:
                    lf_str = "%dx" % f
                else:
                    lf_str = "%2.1fx" % f
                ax1.text((1.0/f)*out_up, out_up*offset, lf_str, color=c,
                         fontsize=linefactor_size)
                ax1.text(out_up*offset, (1.0/f)*out_up, lf_str, color=c,
                         fontsize=linefactor_size)
    
    
        #######
        #  Scatter
        grey_idx = list()
        timeout_x = list()
        timeout_y = list()
        timeout_both = list()
        rest_idx = list()
        for idx_x, x in enumerate(x_data):
            if x >= max_val > y_data[idx_x]:
                # timeout of x algo
                timeout_x.append(idx_x)
            elif y_data[idx_x] >= max_val > x:
                # timeout of y algo
                timeout_y.append(idx_x)
            elif y_data[idx_x] >= max_val and x >= max_val:
                # timeout of both algos
                timeout_both.append(idx_x)
            elif y_data[idx_x] < grey_factor*x and x < grey_factor*y_data[idx_x]:
                grey_idx.append(idx_x)
            else:
                rest_idx.append(idx_x)
    
        # Regular points
        if len(grey_idx) > 1:
            ax1.scatter(x_data[grey_idx], y_data[grey_idx], marker=grey_marker,
                        c=c_good_points)
        ax1.scatter(x_data[rest_idx], y_data[rest_idx], marker=regular_marker,
                    c=c_other_points)
    
        # max_val lines
        ax1.plot([maximum_value, maximum_value], [auto_min_val, maximum_value],
                 c=c_other_points, linestyle="--", zorder=0, linewidth=size)
        ax1.plot([auto_min_val, maximum_value], [maximum_value, maximum_value],
                 c=c_other_points, linestyle="--", zorder=0, linewidth=size)
    
        # Timeout points
        ax1.scatter([timeout_val]*len(timeout_x), y_data[timeout_x],
                    marker=timeout_marker, c=c_other_points)
        ax1.scatter([timeout_val]*len(timeout_both), [timeout_val]*len(timeout_both),
                    marker=timeout_marker, c=c_other_points)
        ax1.scatter(x_data[timeout_y], [timeout_val]*len(timeout_y),
                    marker=timeout_marker, c=c_other_points)
    
        # Plot timeout line
    #    ax1.plot([timeout_val, timeout_val], [auto_min_val, timeout_val],
    #             c=c_other_points, linestyle=":", zorder=0)
    #    ax1.plot([auto_min_val, timeout_val], [timeout_val, timeout_val],
    #             c=c_other_points, linestyle=":", zorder=0)
    
        if debug:
            # debug option
            ax1.scatter(x_data, y_data, marker="o", facecolor="", s=50,
                        label="original data")
    
        # Set axes scale and limits
        ax1.set_xscale("log")
        ax1.set_yscale("log")
    
        # Set axes labels
        ax1.set_xlabel(labels[0], fontsize=label_size)
        ax1.set_ylabel(labels[1], fontsize=label_size)
    
        if debug:
            # Plot legend
            leg = ax1.legend(loc='best', fancybox=True)
            leg.get_frame().set_alpha(0.5)
    
        # Save or show figure
        tight_layout()
        subplots_adjust(top=0.85)
        
        max_val = timeout_val*2
        auto_min_val *= 0.9
        ax1.set_autoscale_on(False)
        if max_val is not None and min_val is None:
            # User sets max val
            ax1.set_ylim([auto_min_val, max_val])
            ax1.set_xlim(ax1.get_ylim())
        elif max_val > min_val and max_val is not None and min_val is not None:
            # User sets both, min and max -val
            ax1.set_ylim([min_val, max_val])
            ax1.set_xlim(ax1.get_ylim())
        else:
            # User sets nothing
            ax1.set_xlim([auto_min_val, max_val])
            ax1.set_ylim(ax1.get_xlim())
    
        # Plot maximum value as tick
        if int(maximum_value) == maximum_value:
            maximum_value = int(maximum_value)
            maximum_str = r"$%d$" % maximum_value
        else:
            maximum_str = r"$%5.2f$" % maximum_value
    
        if int(np.log10(maximum_value)) != np.log10(maximum_value):
            # If we do not already have this ticklabel as a regular label
            ax1.text(ax1.get_ylim()[0] - 0.1 * np.abs(ax1.get_ylim()[0]),
                     maximum_value,
                     maximum_str,
                     horizontalalignment='right', verticalalignment="center",
                     fontsize=user_fontsize)
            ax1.text(maximum_value,
                     ax1.get_ylim()[0] - 0.1 * np.abs(ax1.get_ylim()[0]),
                     maximum_str,
                     horizontalalignment='center', verticalalignment="top",
                     fontsize=user_fontsize)
    
        # Plot 'timeout'
        ax1.text(ax1.get_xlim()[0] - 0.1 * np.abs(ax1.get_ylim()[0]),
                 timeout_val,
                 "timeout ", horizontalalignment='right',
                 verticalalignment="center", fontsize=user_fontsize)
        ax1.text(timeout_val,
                 ax1.get_ylim()[0] - 0.1 * np.abs(ax1.get_ylim()[0]),
                 "timeout ",  horizontalalignment='center', verticalalignment="top",
                 fontsize=user_fontsize, rotation=45)
    
        #########
        # Adjust ticks > max_val
        ax1.xaxis.set_ticks_position('bottom')
        ax1.yaxis.set_ticks_position('left')
        # major axes
        for tic in ax1.xaxis.get_major_ticks():
            if tic._loc > maximum_value:
                tic.tick1On = tic.tick2On = False
        for tic in ax1.yaxis.get_major_ticks():
            if tic._loc > maximum_value:
                tic.tick1On = tic.tick2On = False
    
        # minor axes
        for tic in ax1.xaxis.get_minor_ticks():
            if tic._loc > maximum_value:
                tic.tick1On = tic.tick2On = False
        for tic in ax1.yaxis.get_minor_ticks():
            if tic._loc > maximum_value:
                tic.tick1On = tic.tick2On = False
    
        # tick labels
        ticks_x = ax1.get_xticks()
        new_ticks_label = list()
        for l_idx in range(len(ticks_x)):
            if ticks_x[l_idx] < maximum_value:
                new_ticks_label.append(ticks_x[l_idx])
        ax1.set_xticklabels(new_ticks_label)  # , rotation=45)
    
        ticks_y = ax1.get_yticks()
        new_ticks_label = list()
        for l_idx in range(len(ticks_y)):
            if ticks_x[l_idx] < maximum_value:
                if 0 < ticks_x[l_idx] < 1:
                    new_ticks_label.append(str(r"$10^{%d}$" % int(np.log10(ticks_x[l_idx]))))
                if 1 <= ticks_x[l_idx] < 1000:
                    new_ticks_label.append(str(r"$%d^{ }$" % int(ticks_x[l_idx])))
                if 1000 <= ticks_x[l_idx]:
                    new_ticks_label.append(str(r"$10^{%d}$" % int(np.log10(ticks_x[l_idx]))))
        ax1.set_yticklabels(new_ticks_label)  # , rotation=45)
        ax1.set_xticklabels(new_ticks_label)
    
        # Change fontsize for ticklabels
        setp(ax1.get_yticklabels(), fontsize=ticklabel_size)
        setp(ax1.get_xticklabels(), fontsize=ticklabel_size)
        tight_layout()
        if save != "":
            savefig(save, dpi=dpi, facecolor='w', edgecolor='w',
                    orientation='portrait', papertype=None, format=None,
                    transparent=False, pad_inches=0.02, bbox_inches='tight')
        else:
            show()
            
    def speedup_params_plot(self, main_dict):
        p_solvers = { "minisat-HACK-999ED-CSSC": 10,
                        "Riss-4.27" : 214,
                        "YalSAT" : 16,
                        "clasp-3.0.4-p8" : 75,
                        "cryptominisat" : 26,
                        "lingeling": 323,
                        "SparrowToRiss" : 222,
                        "DCCASat+march-rw" : 1,
                        "CSCCSat2014" : 3,
                        "probSAT" : 9,
                        "gnoveltyGCwa" : 2,
                        "gnoveltyGCa" : 2,
                        "gnoveltyPCL" : 5, 
                        "forlnodrup" : 44,
                        "simpsat" : 5,
                        "SolverFourtyThree": 12,
                        "rissgExt" : 193,
                        "rissg" : 125,
                        "satj": 10,
                        "clasp-cssc": 83
                      }
        if self.args.year == 2013:
            p_solvers["lingeling"] = 141
        # track 
        #       -> scenario 
        #         -> solver_dict 
        #           -> configurator/default/default-test 
        #             ->  "training_perf"/"test_perf" 
        #               -> "PAR1"/"PAR10"/"TOs"/"Performance"
        
        
        solver_points_dict = {}
        for track, track_dict in main_dict.iteritems():
            for scen, scen_dict in track_dict.iteritems():
                for solver, solver_dict in scen_dict.iteritems():
                    solver_points_dict[solver] = solver_points_dict.get(solver, {"x":[], "y":[]})
                    best_configurator = self.get_best_train(solver_dict)
                    best_run = solver_dict[best_configurator]
                    default = solver_dict["default"]["Performance"]
                    configured = best_run["test_perf"]["Performance"]
                    default, configured = self.extract_values_in_order(default, configured, default.keys()) 
                    pairs = filter(lambda x: x[0] < self.cutoff or x[1] < self.cutoff, zip(default, configured))
                    #pairs = zip(default, configured)
                    if pairs: # if all instances were unsolved, skip
                        default = sum(map(lambda x: x[0] if x[0] < self.cutoff else self.cutoff*self.args.parfactor, pairs))
                        configured = sum(map(lambda x: x[1] if x[1] < self.cutoff else self.cutoff*self.args.parfactor, pairs))
                        speedup = default/configured
                        if speedup < 1:
                            print("%s %s %s def: %.2f conf: %.2f" %(track, scen, solver, default, configured))
                        solver_points_dict[solver]["x"].append(p_solvers[solver])
                        solver_points_dict[solver]["y"].append(speedup)
                    
        import numpy as np
        import matplotlib.pyplot as plt
        import matplotlib as mpl
        
        ax = plt.subplot(1,1,1)
        cmap = ["#a50026","#d73027","#f46d43","#fdae61","#fee090","#ffffbf","#e0f3f8","#abd9e9","#74add1","#4575b4","#313695"]
        cmap.reverse()
        markers = ["o", "s", "v", "D"]
        i = 0
        sol = solver_points_dict.keys()
        sol_n_pairs = map(lambda x: (x,p_solvers[x]), sol)
        sol_n_pairs.sort(key=lambda x: x[1], reverse=False)
        for solver in sol_n_pairs:
            xy = solver_points_dict[solver[0]]
            ax.scatter(xy["x"],xy["y"], label=solver[0].rstrip("-CSSC"), c=cmap[i], marker=markers[i%4], s=60)
            i += 1
        font = {"size": 20}
        matplotlib.rc('font', **font)
        ax.set_yscale('log')
        ax.set_xscale('log')
        ax.set_ylabel("Speedup")
        ax.set_xlabel("#Parameters")
        ax.set_xlim([0.9,1000])
        ax.set_ylim([0.1,2000])
        handles, labels = ax.get_legend_handles_labels()
        lgd = ax.legend(handles, labels, loc='upper center', bbox_to_anchor=(0.5,-0.11), ncol=2)
        ax.grid('on')
        plt.savefig("speedup_params.pdf", format="pdf", bbox_inches='tight')
        
if __name__ == '__main__':
    
    logging.basicConfig(level="DEBUG")
    #logging.basicConfig(level="INFO")
    
    parser = argparse.ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    
    parser.add_argument("-d", "--dir", default=".", help="directory with csv files")
    parser.add_argument("-s", "--scatter", action="store_true", default=False, help="Build Scatter Plots")
    parser.add_argument("-t", "--tables", action="store_true", default=False, help="Build Tables")
    parser.add_argument("-p", "--permutationtest", action="store_true", default=False, help="permutation test for highlighting in tables")
    parser.add_argument("-S", "--speeduptables", action="store_true", default=False, help="Build Speedup Tables")
    parser.add_argument("-a", "--allspeeduptables", action="store_true", default=False, help="Build all Speedup Tables")
    parser.add_argument("-y", "--year", type=int, default=2014, help="Competition year")
    parser.add_argument("--plotspeedup", action="store_true", default=False,help="Plot Speedup plots")
    parser.add_argument("--parfactor", type=int, default=10, help="PAR-X Factor in speedup plots")
    
    args_ = parser.parse_args()
    
    eva = EvaluateCSSC(args_)
    eva.main()
